#!/usr/bin/env python

"""
View annotated sources.

Copyright (C) 2006 Paul Boddie <paul@boddie.org.uk>

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this library; see the file LICENCE.txt
If not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""

from compiler.visitor import ASTVisitor
import sys

class Viewer(ASTVisitor):

    """
    A viewing visitor for AST nodes.
    """

    def __init__(self, stream):
        ASTVisitor.__init__(self)
        self.cached_files = {}
        self.printed_lines = {}
        self.visitor = self
        self.stream = stream

    def process(self, module):
        self.dispatch(module)

    def dispatch(self, node):
        self.print_line(getattr(node, "filename", None), getattr(node, "lineno", None))
        ASTVisitor.dispatch(self, node)

    def print_line(self, filename, lineno):
        last_printed = self.printed_lines.get(filename, 0)
        if lineno > last_printed:
            self.stream.write(self.get_line(filename, lineno))
            self.printed_lines[filename] = lineno

    def get_line(self, filename, lineno):
        if filename is None or lineno is None:
            return ""

        if self.cached_files.has_key(filename):
            lines = self.cached_files[filename]
        else:
            f = open(filename)
            try:
                self.cached_files[filename] = lines = f.readlines()
            finally:
                f.close()

        try:
            return lines[lineno - 1]
        except IndexError:
            return ""

# Convenience functions.

def view(module, stream=None):
    viewer = Viewer(stream or sys.stdout)
    viewer.process(module.original)

# vim: tabstop=4 expandtab shiftwidth=4
